/*******************************************************************/
/* Comptabilit Standard                                           */
/* 01/01/2002 - Version 0.1                                        */
/* CONRAD Pascal                                                   */
/*-----------------------------------------------------------------*/
/* Module de gestion du tableau d'affichage des lignes             */
/* 01/03/02                                                        */
/*******************************************************************/

#ifndef __LCPT_TAB_H__
#define __LCPT_TAB_H__


#include <gdk/gdk.h>
#include <gtk/gtkwidget.h>
#include <gtk/gtkadjustment.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define LCPT_TAB_COLUMN_CODE   -1
#define LCPT_TAB_COLUMN_DATE    0
#define LCPT_TAB_COLUMN_LIBELLE 1
#define LCPT_TAB_COLUMN_PIECE   2
#define LCPT_TAB_COLUMN_COMPTE  3
#define LCPT_TAB_COLUMN_DEBIT   4
#define LCPT_TAB_COLUMN_CREDIT  5 

#define TYPE_LCPT_TAB            (lcpt_tab_get_type ())
#define LCPT_TAB(obj)            (GTK_CHECK_CAST ((obj),TYPE_LCPT_TAB,LcptTab))
#define LCPT_TAB_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass),TYPE_LCPT_TAB, LcptTabClass))
#define IS_LCPT_TAB(obj)         (GTK_CHECK_TYPE ((obj), TYPE_LCPT_TAB))
#define IS_LCPT_TAB_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_LCPT_TAB))


typedef struct _LcptTab       LcptTab;
typedef struct _LcptTabClass  LcptTabClass;
typedef struct _laccdata      laccdata;
typedef struct _lacc          lacc;
typedef struct _cell          lacccell;
typedef struct _edit          Edit;

struct _cell
{
  gint Px,Ta,PTxt;
};

struct _edit
{
  gchar str[255];
  gint  max_length;
  gint  Xoffset;
  gint  current_pos;
  gint  length_txt;
  gint  PosCur;
  gboolean Modify;
};

struct _laccdata
{
  char Code[11];
  char Date[11];
  char Lib[31];
  char Piece[11];
  char Compte[11];
  double Debit,Credit;
  char DebTxt[20];
  char CreTxt[20];
};

struct _lacc
{
  int            Id;
  gboolean       EmptyRow;
  laccdata       D;
  lacc           *Su,*Pr;
};

struct _LcptTab
{
  GtkWidget      widget;

  lacc           *ORG_ACC ;
  lacc           *Selection;

  GdkWindow      *Title;
  gint           Title_width;
  gint           Title_height;

  GdkWindow      *Row;
  gint           List_width;
  gint           List_height;

  lacccell       Cell[6];
  Edit           E;

  GtkAdjustment  *VAdj;
  GdkFont        *Fonte;
  GdkGC          *Crayon;
  gint            Up,Down,MaxRow,RowSel,ColumnSel;
  gint           RowHeight;
  gint           NbRowDisplay;
  gboolean       Freeze;
  gboolean       AsMoved;  /* TRUE si dplacer par programmation externe */
};

struct _LcptTabClass
{
  GtkWidgetClass parent_class;

  void  (*set_scroll_adjustments) (LcptTab        *LTab,
				   GtkAdjustment  *hadjustment,
				   GtkAdjustment  *vadjustment);
  gboolean (*lcpt_change) (LcptTab *LTab);
  gboolean (*lcpt_insert) (LcptTab *LTab);
  gboolean (*lcpt_change_column) (LcptTab *LTab,gint column);
  gboolean (*lcpt_change_row) (LcptTab *LTab);
  gboolean (*lcpt_change_row_after) (LcptTab *LTab);
  gboolean (*lcpt_insert_text) (LcptTab *LTab,gint column,char *Str,char *Text);
  void (*lcpt_ecart) (gfloat Ecart);
};


GtkType    lcpt_tab_get_type   (void);
GtkWidget* lcpt_tab_new        (void);
void       lcpt_tab_size       (LcptTab *LTab,gint width,gint height);
void       lcpt_tab_row_new(LcptTab *LTab,char *Cde,char *D,char *L,char *P,char *C,double Deb,double Cre);
void       lcpt_tab_row_empty_new(LcptTab *LTab);
void       lcpt_tab_row_get_data(LcptTab *LTab,char *Cde,char *D,char *L,char *P,char *C,double *Deb,double *Cre);
void       lcpt_tab_row_set_data(LcptTab *LTab,char *Cde,char *D,char *L,char *P,char *C,double *Deb,double *Cre);
void       lcpt_tab_row_delete_all(LcptTab *LTab);
void       lcpt_tab_row_delete(LcptTab *LTab);
void       lcpt_tab_freeze(LcptTab *LTab);
void       lcpt_tab_thaw(LcptTab *LTab);
void       lcpt_tab_change_column(LcptTab *LTab,int Column);
void       lcpt_tab_insert_text(LcptTab *LTab,gchar *Str);
void       lcpt_tab_delete_text(LcptTab *LTab,gint Sens,gint NbCar);
gboolean   lcpt_tab_row_search(LcptTab *LTab,gint Column,gchar *Value);
void       lcpt_tab_row_valid(LcptTab *LTab);
#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __LCPT_TAB_H__ */
