/*******************************************************************/
/* Comptabilit Analytique Professionnel                           */
/* 01/01/2002 - Version 0.1                                        */
/* CONRAD Pascal                                                   */
/*-----------------------------------------------------------------*/
/* 08/05/02 - Procdure d'impression PostScript                    */
/* Fichier source                                                  */
/*******************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <dirent.h>
#include <sys/types.h>
#include <gtk/gtk.h>

#include "common.h"
#include "print.h"

FILE *OutPs;
static gchar FontName[64],FontNameBase[64];
static gint Xpos,Ypos,Page,MaxPage;
static gint FontSize,FontWidth,FontHeight,TypeFace,Justification;
static double DFontWidth,DFontHeight;
static gint PageWidth,PageHeight,PaperSize;
static gint LeftMargin,RightMargin,TopMargin,BottomMargin;
/* Indique les fontes dj changer pour l'ISOLatin1 ***/
static gchar FontChanged[20][64];
static gint MaxFont;

/* Function call every time a new page is create */
void (*PS_NewPageExternBefore) (int Page);
void (*PS_NewPageExternAfter) (int Page);

struct
{
  gint Car;
  gint Width;
  char Name[32];
} CarSize[255];

struct {
  char Car;
  char Name[32];
} Encode[9] ={{'',"ecircumflex"},
	      {'',"aacute"},
	      {'',"icircumflex"},
	      {'',"uacute"},
	      {'',"egrave"},
	      {'',"ocircumflex"},
	      {'',"degree"},
	      {'',"edieresis"},
	      {'',"ccdilla"}};

char *InsLoc[]={"FontName","IsFixedPitch","FamilyName","StartCharMetrics",
		"EndCharMetrics","FontBBox","Weight","ItalicAngle"};
#define NB_INS 8;

/* Protos local */
gint LocateIns(char *Ins[],int Nb,gchar *Test);
void InitAFM(void);
void ReadAFMChar(FILE *Fp);
char *LoadInt(gchar *Src,int *Ret);
gchar *Space(gchar *Ptr);
gchar *LoadWord(gchar *Src,gchar *Dest);
int CalculXpos(int x,int Size);
void InitEnd(void);
void InitHeader(gchar *Title);
void SearchFont(gchar *Name);
void ValorByIndex(gchar *Str,int Id,char Separator,gchar *Ret);
void ReadAFM(gchar *N);
int CalculYpos(gint Yp);
void Paginate(void);
void CreateFontDir(void);
void LoadInfoAFM(gchar *N,gchar *FtNOpen,gchar *Ret);
gchar *ControlString(gchar *Str);
void InitISOLatin1(char *Fnt);
int SearchFontLatin(char *Ft);

/*****************************************************************/
/* Cration du fichier font.dir                                  */

void LoadInfoAFM(gchar *N,gchar *FtNOpen,gchar *Ret)
{
  gchar Ligne[127],Var[64],*Ps,*Pd;
  gchar FtN[64],FtW[64],FtI[15];
  FILE *Fp;
  int i,Cpt;

  if ((Fp=fopen(N,"rt"))!=NULL) {
    Cpt=0;
    *FtN=0;
    *FtW=0;
    *FtI=0;
    fgets(Ligne,126,Fp);
    if (strncmp(Ligne,"StartFontMetrics",16)==0) {
      while (!feof(Fp)) {
				if (fgets(Ligne,126,Fp)) {
					Ps=Ligne;
					Pd=Var;
					while (*Ps==32) Ps++;
					if (*Ps!='\n') {
						while ((*Ps!=' ') && (*Ps!='\n')) *Pd++=*Ps++;
						*Pd='\0';
						i=LocateIns(InsLoc,8,Var);
						while (*Ps==32) Ps++;
						Pd=NULL;
						if (i==2) {       /* FamilyName */
							Pd=FtN;
							Cpt++;
						}
						if (i==6) {       /* Weight */
							Pd=FtW;
							Cpt++;
						}
						if (i==7) {       /* ItalicAngle */
							Pd=FtI;
							Cpt++;
						}
						if (Pd!=NULL) {
							while ((*Ps!=' ') && (*Ps!='\n')) *Pd++=*Ps++;
							*Pd=0;
						}
					} /* \n */
				} /* fgets */
				if (Cpt==3) break;
			} /* while */
    } /* StartFontMetrics */
    fclose(Fp);
  }
  if (*FtI!='0') *FtI='i'; else *FtI='r';
  FtI[1]=0;
  sprintf(Ret,"%s -adobe-%s-%s-%s-normal--0-0-0-0-p-0-iso8859-1",FtNOpen,FtN,FtW,FtI);
}


void CreateFontDir(void)
{
  DIR *Dir;
  struct dirent *Ent;
  gchar Name[512],Line[100],fontsdir[512],HomeDir[64];
  FILE *Fp;

  //  sprintf(fontsdir,"%s/fonts.dir",AfmDir);
  strcpy(HomeDir,g_get_home_dir());
  sprintf(fontsdir,"%s/fonts.dir",HomeDir);
  if ((Fp=fopen(fontsdir,"wt"))) {
    if ((Dir=opendir(AfmDir))) {
      while ((Ent=readdir(Dir))) {
				if (*Ent->d_name!='.') {
					sprintf(Name,"%s/%s",AfmDir,Ent->d_name);
					LoadInfoAFM(Name,Ent->d_name,Line);
					fputs(Line,Fp);
					fputc('\n',Fp);
				}
      }
      closedir(Dir);
    }
    fclose(Fp);
  }
}

/*****************************************************************/
gint LocateIns(char *Ins[],int Nb,gchar *Test)
{
  gint i;

  i=0;
  while (i<Nb) {
    if (strcmp(Ins[i],Test)==0) return i;
    i++;
  }
  return (-1);
}

gchar *Space(gchar *Ptr)
{
  while ((*Ptr==32) || (*Ptr==9)) Ptr++;
  return (Ptr);
}

gchar *LoadWord(gchar *Src,gchar *Dest)
{
  Src=Space(Src);
  while ((isalnum(*Src)) || (*Src=='_')) *Dest++=*Src++;
  *Dest=0;
  return (Src);
}

gchar *LoadInt(gchar *Src,int *Ret)
{
  gchar Tmp[15],*Dest;
  int Sgn,V;

  Src=Space(Src);
  Sgn=1;
  if (*Src=='-') {
    Sgn=-1;
    Src++;
  }
  Dest=Tmp;
  V=0;
  while (isdigit(*Src)) V=(V*10)+(*Src++-48);
  V=V*Sgn;
  *Ret=V;
  return (Src);
}

/* Lecture de la table des caractres AFM **************************************/
void ReadAFMChar(FILE *Fp)
{
  gchar Ligne[127],Var[64],*Ptr;
  int Quit,Car,CarWidth,i;

  Quit=FALSE;
  do {
    if (fgets(Ligne,127,Fp)) {
      Ptr=Ligne;
      while (*Ptr==32) Ptr++;
      if (*Ptr=='C') {
				Ptr++;
				Ptr=LoadInt(Ptr,&Car);
				Ptr=Space(Ptr);
				if (*Ptr==';') {
					Ptr++;
					Ptr=LoadWord(Ptr,Var);
					if (strcmp(Var,"WX")==0) {
						Ptr=LoadInt(Ptr,&CarWidth);
						Ptr=Space(Ptr);
						if (*Ptr==';') {
							Ptr++;
							Ptr=Space(Ptr);
							if ((*Ptr=='N') && (*Ptr+1==' ')) {
								Ptr++;
								Ptr=LoadWord(Ptr,Var);
							}
						}
					}
				}
				if (Car==-1) {
					for (i=0;i<9;i++) {
						if (strcmp(Var,Encode[i].Name)==0) Car=Encode[i].Car;
					}
				}
				if (Car!=-1)  {
					CarSize[Car].Car=Car;
					CarSize[Car].Width=CarWidth;
					strcpy(CarSize[Car].Name,Var);
				}
      } else {
				LoadWord(Ptr,Var);
				if (strcmp(Var,"EndCharMetrics")==0) Quit=TRUE;
			}
    } else Quit=TRUE;
  } while (!Quit);
}

/* Initialisation des caractres et conversion pour les fontes AFM*******/
void InitAFM(void)
{
  int i;

  for (i=0;i<255;i++) {
    CarSize[i].Car=0;
    CarSize[i].Width=0;
    *CarSize[i].Name='\0';
  }
}

void ReadAFM(gchar *N)
{
  gchar Ligne[127],Var[64],*Ps,*Pd;
  FILE *Fp;
  int i;

  InitAFM();
  if ((Fp=fopen(N,"rt"))!=NULL) {
    while (!feof(Fp)) {
      if (fgets(Ligne,126,Fp)) {
				Ps=Ligne;
				Pd=Var;
				while (*Ps==20) Ps++;
				if (*Ps!='\n') {
					while ((*Ps!=' ') && (*Ps!='\n')) *Pd++=*Ps++;
					*Pd='\0';
					i=LocateIns(InsLoc,8,Var);
					if (i==0) {
						Ps=Space(Ps);
						Pd=FontName;
						while ((*Ps!=0) && (*Ps!='\n') && (*Ps!=32)) *Pd++=*Ps++;
						*Pd=0;
					}
					if (i==3) {       /* StartCharMetrics */
						ReadAFMChar(Fp);
						break;
					}
					if (i==5) {       /* FontBBox */
						Ps=LoadInt(Ps,&i);
						Ps=LoadInt(Ps,&i);
						Ps=LoadInt(Ps,&FontWidth);
						Ps=LoadInt(Ps,&FontHeight);
					}
				}
      }
    }
    fclose(Fp);
  }
}

/* Recherche d'une valeur dans une chaine dans fonts.dir ***********************/
void ValorByIndex(gchar *Str,int Id,char Separator,gchar *Ret)
{
  int i;

  i=0;
  while ((*Str) && (i!=Id)) {
    while ((*Str) && (*Str!=Separator)) Str++;
    if (*Str==Separator) {
      i++;
      Str++;
    }
  }
  if (*Str) {
    while ((*Str) && (*Str!=Separator)) *Ret++=*Str++;
  }
  *Ret=0;
}

void SearchFont(gchar *Name)
{
  gchar RepFnt[512],Ligne[128],File[64],*s,*d,HomeDir[64];
  gchar DataTmp[255];
  FILE *Fp;
  int Ok,Ok1,Ok2;

  Ok=FALSE;
  sprintf(RepFnt,"%s/fonts.dir",AfmDir);
  if ((Fp=fopen(RepFnt,"rt"))==NULL) {
    strcpy(HomeDir,g_get_home_dir());
    sprintf(RepFnt,"%s/fonts.dir",HomeDir);
    if ((Fp=fopen(RepFnt,"rt"))==NULL) {
      CreateFontDir();
      Fp=fopen(RepFnt,"rt");
    }
  }
  if (Fp!=NULL) {
    //g_print("Recherche :%s TypeFace %i\n",Name,TypeFace);
    while ((!feof(Fp)) && (Ok==FALSE)) {
      if (fgets(Ligne,128,Fp)) {
				ValorByIndex(Ligne,2,'-',DataTmp);
				if (strcasecmp(DataTmp,Name)==0) {
					Ok=FALSE;
					ValorByIndex(Ligne,3,'-',DataTmp);
					if ((TypeFace & FNT_BOLD)!=0) {
						if ((strcmp(DataTmp,"Bold")==0) || (strcmp(DataTmp,"Demibold")==0)) Ok1=TRUE; else Ok1=FALSE;
					} else {
						if ((strcmp(DataTmp,"Medium")==0) || (strcmp(DataTmp,"Light")==0)) Ok1=TRUE; else Ok1=FALSE;
					}
					ValorByIndex(Ligne,4,'-',DataTmp);
					if ((TypeFace & FNT_ITALIC)!=0) {
						if ((strcmp(DataTmp,"i")==0) || (strcmp(DataTmp,"o")==0)) Ok2=TRUE;else Ok2=FALSE;
					} else {
						if ((strcmp(DataTmp,"i")==0) || (strcmp(DataTmp,"o")==0)) Ok2=FALSE;else Ok2=TRUE;
					}
					if (Ok1 && Ok2) Ok=TRUE;
				}
      }
    }
    fclose(Fp);
    if (Ok) {
      ValorByIndex(Ligne,0,'-',DataTmp);
      //      g_print("Trouve le fichier %s",DataTmp);
      d=File;
      s=DataTmp;
      while (*s!='.') *d++=*s++;
      *d++='\0';
      strcat(File,".afm");
      sprintf(RepFnt,"%s/%s",AfmDir,File);
      ReadAFM(RepFnt);
    } else g_print("Fonte %s non trouv !\n",Name);
  }
}

/* Ecriture de l'entte du fichier PostScript **********************************/
void InitHeader(gchar *Title)
{
  gchar PsTmp[512];

  fputs("%!PS-Adobe-2.0\n",OutPs);
  fputs("%%Creator: LinCompta 0.3 (CONRAD Pascal)\n",OutPs);
  sprintf(PsTmp,"%%%%Title: %s\n",Title);
  fputs(PsTmp,OutPs);
  sprintf(PsTmp,"%%%%BoundingBox: 0 0 %i %i\n",PageWidth,PageHeight);
  fputs(PsTmp,OutPs);
  if (PaperSize==1) fputs("%%DocumentPaperSizes: a4\n",OutPs); else
    fputs("%%DocumentPaperSizes: letter\n",OutPs);
  fputs("%%EndComments\n",OutPs);
  fputs("%%BeginProlog\n",OutPs);
  fputs("/FSD {findfont exch scalefont setfont} bind def\n",OutPs);
  fputs("/MS {moveto show} bind def\n",OutPs);
  fputs("%%EndProlog\n",OutPs);
  fputs("%%BeginSetup\n",OutPs);
  fputs("<<\n",OutPs);
  sprintf(PsTmp,"  /PageSize [%i %i]\n",PageWidth,PageHeight);
  fputs(PsTmp,OutPs);
  fputs("  /Orientation 0\n",OutPs);
  fputs(">> setpagedevice\n",OutPs);
  /* ISOLatin1Encoding */
  fputs("%%EndSetup\n",OutPs);
  fputs("%%BeginDocument\n",OutPs);
  sprintf(PsTmp,"%%%%Page: %i %i\n",Page,MaxPage);
  fputs(PsTmp,OutPs);
}

/* Ecriture de la fin du fichier PostScript ************************************/
void InitEnd(void)
{
  fputs("%%PageTrailer\n",OutPs);
  fputs("showpage\n",OutPs);
  fputs("%%Trailer\n",OutPs);
  fputs("%%EndDocument\n",OutPs);
  fputs("%%EOF\n",OutPs);
}

/************************************************************/
/* Gestion des fontes  craractres franaise ***************/

int SearchFontLatin(char *Ft)
{
  int i,Ret;

  Ret=FALSE;
  i=0;
  while ((i<MaxFont) && (Ret==FALSE)) {
    if (strcmp(FontChanged[i],Ft)==0) Ret=TRUE;
    i++;
  }
  return (Ret);
}

void InitISOLatin1(char *Fnt)
{
  gchar Tmp[127];

  strcpy(FontChanged[MaxFont++],Fnt);
  sprintf(Tmp,"/%s findfont\n",Fnt);
  fputs(Tmp,OutPs);
  fputs("dup length dict begin\n",OutPs);
  fputs("  { 1 index /FID ne\n",OutPs);
  fputs("     {def}\n",OutPs);
  fputs("     {pop pop}\n",OutPs);
  fputs("     ifelse\n",OutPs);
  fputs("  } forall\n",OutPs);
  fputs("  /Encoding ISOLatin1Encoding def\n",OutPs);
  fputs("  currentdict\n",OutPs);
  fputs("end\n",OutPs);
  sprintf(Tmp,"/%s-ISOLatin1 exch definefont pop\n",Fnt);
  fputs(Tmp,OutPs);
}

int CalculXpos(int x,int Size)
{
  int Xret;

  Xret=x;
  switch (Justification)
    {
    case LEFT_JUST : Xret=x;
      break;
    case CENTER_JUST : Xret-=(Size/2);
      break;
    case RIGHT_JUST : Xret-=Size;
      break;
    }
  return (Xret);
}

gint CalculYpos(gint Yp)
{
  Yp=PageHeight-Yp;
  return (Yp);
}

/* Pagination correcte des feuilles ********************************************/
/* Lecture du fichier d'impression et rcriture */
void Paginate(void)
{
  FILE *FpIn,*FpOut;
  gchar Line[127],PsTmp[512];

  Page=1;
  if ((FpIn=fopen("/tmp/lcpt_print.ps","rt"))!=NULL) {
    if ((FpOut=fopen("/tmp/lcpt_print.ps~","wt"))!=NULL) {
      while (!feof(FpIn)) {
				if (fgets(Line,127,FpIn)) {
					if (strncmp(Line,"%%Page:",7)==0) {
						sprintf(PsTmp,"%%%%Page: %i %i\n",Page,MaxPage);
						fputs(PsTmp,FpOut);
						Page++;
					} else fputs(Line,FpOut);
				}
      }
      fclose(FpOut);
      unlink("/tmp/lcpt_print.ps");
      rename("/tmp/lcpt_print.ps~","/tmp/lcpt_print.ps");
    }
    fclose(FpIn);
  }
}

/* Controle des caractres spciaux PostScript *********************************/
/* et place le texte dans un nouveau buffer ************************************/
gchar *ControlString(gchar *Str)
{
  gchar *P,*CtTmp;
	int Ta,TaMax;

	TaMax=512;
	Ta=0;
	CtTmp=(gchar *) malloc(TaMax);
  P=CtTmp;
  while (*Str) {
    if ((*Str=='(') || (*Str=='\\') || (*Str==')')) {
			*P++='\\';
			Ta++;
		}
    *P++=*Str++;
		Ta++;
		if (Ta>=(TaMax-2)) {
			*P=0;
			TaMax+=256;
			CtTmp=realloc(CtTmp,TaMax);
			P=CtTmp;
			while (*P) P++;
		}
  }
  *P=0;
  return (CtTmp);
}

/*********************************************************/
/* PUBLIC ************************************************/
void PS_sendReturn(void)
{
  double a;

  Xpos=LeftMargin;
  a=(double) Ypos+((DFontHeight*FontSize/1000)*1.2);  /* Coef de 1.2 sur l'interligne */
  Ypos=(int) a;
  if (Ypos>(PageHeight-BottomMargin)) {
    Ypos=0+TopMargin;
    PS_sendEOP();
  }
}

void PS_sendEOP(void)
{
  gchar PsTmp[512];

	if (PS_NewPageExternBefore) PS_NewPageExternBefore(Page);
  fputs("%%PageTrailer\n",OutPs);
  fputs("showpage\n",OutPs);
  Page++;
  MaxPage++;
  Xpos=LeftMargin;
  Ypos=0+TopMargin;
  sprintf(PsTmp,"%%%%Page: %i %i\n",Page,MaxPage);
  fputs(PsTmp,OutPs);
  if (PS_NewPageExternAfter) PS_NewPageExternAfter(Page);
}

double PS_CharSize(int Car)
{
  double a;

  a=(double) CarSize[Car].Width;
  a=a*FontSize/1000;
  return (a);
}

int PS_strwidth(gchar *Str)
{
  double i;

  i=0;
  while (*Str) {
    i+=PS_CharSize(*Str);
    Str++;
  }
  return ((int) i);
}

void PS_setFont(gchar *N,int size,int Att)
{
  double w,h;
  gchar PsTmp[512];

  strcpy(FontName,N);
  strcpy(FontNameBase,N);
  TypeFace=Att;
  SearchFont(N);
  if (!SearchFontLatin(FontName)) InitISOLatin1(FontName);
  strcat(FontName,"-ISOLatin1");
  FontSize=size;
  /***/
  w=(double) FontWidth;
  h=(double) FontHeight;
  DFontWidth=w;
  DFontHeight=h;
  /***/
  sprintf(PsTmp,"%i /%s FSD\n",FontSize,FontName);
  //  sprintf(PsTmp,"/%s findfont %i scalefont setfont\n",N,size);
  fputs(PsTmp,OutPs);
}

void PS_setTypeface(int Tpf)
{
  gchar PsTmp[512];

  TypeFace=Tpf;
  SearchFont(FontNameBase);
  if (!SearchFontLatin(FontName)) InitISOLatin1(FontName);
  strcat(FontName,"-ISOLatin1");
  sprintf(PsTmp,"%i /%s FSD\n",FontSize,FontName);
  //  sprintf(PsTmp,"/%s findfont %i scalefont setfont\n",FontName,FontSize);
  fputs(PsTmp,OutPs);
}

void PS_setTextSize(int Size)
{
  double w,h;
  gchar PsTmp[512];

  FontSize=Size;
  w=(double) FontWidth;
  h=(double) FontHeight;
  DFontWidth=w;
  DFontHeight=h;
  sprintf(PsTmp,"%i /%s FSD\n",FontSize,FontName);
  //  sprintf(PsTmp,"/%s findfont %i scalefont setfont\n",FontName,FontSize);
  fputs(PsTmp,OutPs);
}

void  PS_justification(int Justif)
{
  Justification=Justif;
}

void PS_print(gchar *Str)
{
  int Ta,Yp;
  gchar *PsTmp,*CtTmp;

	if (Str!=NULL) {
		Ta=PS_strwidth(Str);
		Xpos=CalculXpos(Xpos,Ta);
		Yp=CalculYpos(Ypos);
		CtTmp=ControlString(Str);
		PsTmp=g_strdup_printf("(%s) %i %i MS\n",CtTmp,Xpos,Yp);
		fputs(PsTmp,OutPs);
		Xpos+=Ta;
		free(PsTmp);
		free(CtTmp);
	}
}

void PS_xprint(gint x,gchar *Str)
{
  int Ta,Yp;
  gchar *PsTmp,*CtTmp;

	if (Str!=NULL) {
		x=(float) x /0.3527777;
		Ta=PS_strwidth(Str);
		Xpos=CalculXpos(x,Ta);
		Yp=CalculYpos(Ypos);
		CtTmp=ControlString(Str);
		PsTmp=g_strdup_printf("(%s) %i %i MS\n",CtTmp,Xpos,Yp);
		fputs(PsTmp,OutPs);
		Xpos=x+Ta;
		free(PsTmp);
		free(CtTmp);	
	}
}

void PS_xyprint(gint x,gint y,gchar *Str)
{
  int Ta;
  gchar *PsTmp,*CtTmp;

  if (Str!=NULL) {
		x=(float) x /0.3527777;
		y=(float) y /0.3527777;
		Ypos=y;
		y=CalculYpos(y);
		Ta=PS_strwidth(Str);
		Xpos=CalculXpos(x,Ta);
		CtTmp=ControlString(Str);
		PsTmp=g_strdup_printf("(%s) %i %i MS\n",CtTmp,Xpos,y);
		fputs(PsTmp,OutPs);
		Xpos=x+Ta;
		free(PsTmp);
		free(CtTmp);	
	}
}

void PS_show(gint x,gint y,gchar *Str)
{
  int Ta;
  gchar *PsTmp,*CtTmp;

  if (Str!=NULL) {
		y=CalculYpos(y);
		Ta=PS_strwidth(Str);
		x=CalculXpos(x,Ta);
		CtTmp=ControlString(Str);
		PsTmp=g_strdup_printf("(%s) %i %i MS\n",CtTmp,x,y);
		fputs(PsTmp,OutPs);
		free(PsTmp);
		free(CtTmp);	
	}
}

/* Impresssion d'un bloc de texte ************************************/
void PS_blockPrint(gint x,gint y,gchar *blk)
{
	gchar *Tmp,*Str,*BeginLine,*BeginWord;
	double Ta,wS;
	gint   RM,Xline;

  x=(float) x /0.3527777;
	y=(float) y /0.3527777;
	Ypos=y;
	Xline=Xpos=x;
	RM=PageWidth-RightMargin;
	BeginLine=BeginWord=Str=blk;
	while (*Str) {
		if (*Str!='\n') {
			if (*Str==32) {
				wS=PS_CharSize(32);
				Str++;
				BeginWord=Str;
			} else {
				wS=0;
				while ((*Str!=32) && (*Str!=0) && (*Str!='\n')) wS+=PS_CharSize(*Str++);
			}			
		  if ((Xpos+(gint) Ta+(gint) wS)>RM) {				// Mot trop grand ?
				Tmp=g_strndup(BeginLine,BeginWord-BeginLine);		// Oui
				PS_show(Xline,Ypos,Tmp);
 				free(Tmp);
				PS_sendReturn();
				Str=BeginWord;					// la boucle P se replace en dbut de mot trop grand
				BeginLine=BeginWord;		// La ligne commence au mot trop grand
				Ta=0;
			} else {									// ca passe
				Ta+=wS;
				BeginWord=Str;
			  Xpos+=Ta;
			}
		} else {							// Saut de page
			if ((Str-BeginLine)!=0) {
				Tmp=g_strndup(BeginLine,(Str-BeginLine)-1);
				PS_show(Xline,Ypos,Tmp);
				free(Tmp);
			}
			PS_sendReturn();
			Str++;
			Ta=0;
			BeginLine=BeginWord=Str;
		}
	}
}

void PS_down(gint y)
{
  int Do;

  Do=(float) y /0.3527777;
  Ypos+=Do;
}

void PS_up(gint y)
{
  int Do;

  Do=(float) y /0.3527777;
  Ypos-=Do;
}

void PS_rectangle(gint x,gint y,gint w,gint h,gint fill)
{
  gchar PsTmp[512];

  x=(float) x /0.3527777;
  y=(float) y /0.3527777;
  y=CalculYpos(y);
	w=(float) w /0.3527777;
	h=(float) h /0.3527777;
  fputs("newpath\n",OutPs);
  sprintf(PsTmp,"%i %i moveto\n",x,y);
  fputs(PsTmp,OutPs);
  sprintf(PsTmp,"%i 0 rlineto\n",w);
  fputs(PsTmp,OutPs);
  sprintf(PsTmp,"0 %i rlineto\n",h*-1);
  fputs(PsTmp,OutPs);
  sprintf(PsTmp,"%i 0 rlineto\n",w*-1);
  fputs(PsTmp,OutPs);
  sprintf(PsTmp,"0 %i rlineto\n",h);
  fputs(PsTmp,OutPs);
  fputs("closepath\n",OutPs);
  fputs("stroke\n",OutPs);
}

void PS_line(gint x1,gint y1,gint x2,gint y2)
{
  gchar PsTmp[512];

  fputs("newpath\n",OutPs);
  sprintf(PsTmp,"%i %i moveto\n",x1,y1);
  fputs(PsTmp,OutPs);
  sprintf(PsTmp,"%i %i lineto\n",x2,y2);
  fputs(PsTmp,OutPs);
  fputs("closepath\n",OutPs);
	fputs("stroke\n",OutPs);
}

/* Retourne la position en hauteur sur la page */
gint PS_getYpos()
{
	float Yp;
	
	Yp=Ypos * 0.3527777;
  return ((gint) Yp);
}

gint PS_getFontHeight()
{
	float a;
	
	a=(DFontHeight*FontSize/1000);
	a*=0.3527777;
	return ((gint) a);
}

gint PS_getStringWidth(gchar *str)
{
	float a;
	
	a=PS_strwidth(str);
	a*=0.3527777;
	return ((gint) a);
}

void PS_beginPrint(int size,char *Title,void (*FuncPageBefore) (),void (*FuncPageAfter) ())
{
  switch (size)
    {
    case 1 : PageWidth=596;    /* A4 */
             PageHeight=842;
             TopMargin=72;
             BottomMargin=72;
             LeftMargin=72;
             RightMargin=72;
             break;
    case 2 : PageWidth=596;    /* letter */
             PageHeight=842;
             TopMargin=72;
             BottomMargin=72;
             LeftMargin=72;
             RightMargin=72;
             break;
    default: PageWidth=596;    /* A4 */
             PageHeight=842;
             TopMargin=72;
             BottomMargin=72;
             LeftMargin=72;
             RightMargin=72;
             break;
    }
  Page=1;
  MaxPage=1;
  Xpos=0+LeftMargin;
  Ypos=0+TopMargin;
  FontSize=12;
  TypeFace=FNT_NORMAL;
  MaxFont=0;
  PS_NewPageExternBefore=FuncPageBefore;
  PS_NewPageExternAfter=FuncPageAfter;
  if ((OutPs=fopen("/tmp/lcpt_print.ps","wt"))!=NULL) {
    InitHeader(Title);
  }
}

void PS_endPrint(char *cmd)
{
  char LprCmd[255];

  InitEnd();
  fclose(OutPs);
  Paginate();
  sprintf(LprCmd,"/usr/bin/%s %s",cmd,"/tmp/lcpt_print.ps"); 
  system(LprCmd);
}
